/*
GNU General Public License

DSTC (Death Star Trash Compactor) - Java ZIP Utility
Copyright (C) 2000  Howard A Kistler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package com.hexidec.tool;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

/** DSTC (Death Star Trash Compactor)
  * Class for manipulating ZIP archives
  *
  * @author Howard Kistler
  * @version 0.1
  *
  * requires Java 2 (JDK 1.3)
  *
  * VERSION HISTORY
  * 0.1 - initially creation (08/21/2000)
  */

public class DSTC
{

	/** Constructor
	  */
	public DSTC(File archive, Vector vcFiles)
	{
		try
		{
			makeZip(archive, vcFiles);
		}
		catch(FileNotFoundException fnfe)
		{
			sysMsg("FileNotFoundException : " + fnfe.getMessage());
			System.exit(1);
		}
		catch(IOException ioe)
		{
			sysMsg("IOException : " + ioe.getMessage());
			System.exit(2);
		}
	}

	public DSTC(String filename, Vector vcFiles)
	{
		this(new File(filename), vcFiles);
	}

	public DSTC(File archive)
	{
		try
		{
			descZip(archive);
		}
		catch(FileNotFoundException fnfe)
		{
			sysMsg("FileNotFoundException : " + fnfe.getMessage());
			System.exit(1);
		}
		catch(IOException ioe)
		{
			sysMsg("IOException : " + ioe.getMessage());
			System.exit(2);
		}
	}

	public DSTC(String filename)
	{
		this(new File(filename));
	}

	public DSTC()
	{
	}

	/** Returns a Vector of ZipEntry objects contained in the specified ZIP archive
	  */
	public Vector scanZip(File archive) throws FileNotFoundException, IOException
	{
		Vector vcScan = new Vector();
		ZipFile zf = new ZipFile(archive);
		Enumeration zipEntries = zf.entries();
		while(zipEntries.hasMoreElements())
		{
			ZipEntry zEntry = (ZipEntry)(zipEntries.nextElement());
			vcScan.add(zEntry);
		}
		zf.close();
		return vcScan;
	}

	public Vector scanZip(String filename) throws FileNotFoundException, IOException
	{
		return this.scanZip(new File(filename));
	}

	/** Describes a ZIP archive to the command line
	  */
	public void descZip(File archive) throws FileNotFoundException, IOException
	{
		ZipFile zf = new ZipFile(archive);
		Enumeration zipEntries = zf.entries();
		while(zipEntries.hasMoreElements())
		{
			ZipEntry zEntry = (ZipEntry)(zipEntries.nextElement());
			sysMsg(verboseDesc(zEntry));
		}
		zf.close();
	}

	public void descZip(String archiveName) throws FileNotFoundException, IOException
	{
		this.descZip(new File(archiveName));
	}

	private String verboseDesc(ZipEntry ze)
	{
		String method = "UNKNOWN";
		if(ze.getMethod() == ZipEntry.DEFLATED) { method = "DEFLATED"; }
		if(ze.getMethod() == ZipEntry.STORED)   { method = "STORED"; }
		int percCompression = (int)(Math.round(100.0f - (ze.getCompressedSize() / (ze.getSize() / 100.0f))));
		String zipDescrip = ze.getName() + " " + percCompression + "% (" + ze.getSize() + "/" + ze.getCompressedSize() + ") [" + method + "] " + ze.getTime();
		return zipDescrip;
	}

	/** Creates a ZIP archive from a Vector of File objects
	  */
	public void makeZip(File archive, Vector vcFiles) throws FileNotFoundException, IOException
	{
		ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(archive));
		for(int i = 0; i < vcFiles.size(); i++)
		{
			File file = (File)(vcFiles.elementAt(i));
			ZipEntry zEntry = new ZipEntry(file.getName());
			FileInputStream fis = new FileInputStream(file);
			zos.putNextEntry(zEntry);
			int datum = fis.read();
			while(datum > -1)
			{
				zos.write(datum);
				datum = fis.read();
			}
			zos.flush();
			zos.closeEntry();
			fis.close();
		}
		zos.close();
	}

	public void makeZip(String filename, Vector vcFiles) throws FileNotFoundException, IOException
	{
		this.makeZip(new File(filename), vcFiles);
	}

	// Currently, this replaces the whole archive with the new file - not good
	// Have to find a way to perform insertions or to stream the existing archive and tack on the new file at the end
	public void addFile(File archive, File addition) throws FileNotFoundException, IOException
	{
		ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(archive));
		ZipEntry zEntry = new ZipEntry(addition.getName());
		FileInputStream fis = new FileInputStream(addition);
		zos.putNextEntry(zEntry);
		int datum = fis.read();
		while(datum > -1)
		{
			zos.write(datum);
			datum = fis.read();
		}
		zos.flush();
		zos.closeEntry();
		fis.close();
		zos.close();
	}

	public void addFile(String archivename, String filename) throws FileNotFoundException, IOException
	{
		this.addFile(new File(archivename), new File(filename));
	}

	/** Extracts all the files from the specified ZIP archive
	  */
	public void extractAllFiles(File archive) throws FileNotFoundException, IOException
	{
		ZipInputStream zis = new ZipInputStream(new FileInputStream(archive));
		while(true)
		{
			ZipEntry zEntry = zis.getNextEntry();
			if(zEntry == null) { break; }
			FileOutputStream fos = new FileOutputStream(new File(zEntry.getName()));
			int datum = zis.read();
			while(datum > -1)
			{
				fos.write(datum);
				datum = zis.read();
			}
			fos.flush();
			fos.close();
			zis.closeEntry();
		}
		zis.close();
	}

	public void extractAllFiles(String archivename) throws FileNotFoundException, IOException
	{
		this.extractAllFiles(new File(archivename));
	}

	/** Extracts the specified file from the specified ZIP archive
	  */
	public void extractFile(File archive, File extract) throws FileNotFoundException, IOException
	{
		ZipInputStream zis = new ZipInputStream(new FileInputStream(archive));
		FileOutputStream fos = new FileOutputStream(extract);
		String filename = extract.getName();
		while(true)
		{
			ZipEntry zEntry = zis.getNextEntry();
			if(zEntry.getName().equals(filename))
			{
				int datum = zis.read();
				while(datum > -1)
				{
					fos.write(datum);
					datum = zis.read();
				}
				break;
			}
		}
		fos.flush();
		fos.close();
		zis.close();
	}

	public void extractFile(String archivename, String filename) throws FileNotFoundException, IOException
	{
		this.extractFile(new File(archivename), new File(filename));
	}

	/** Convenience method for handling message output
	  */
	private void sysMsg(String msg)
	{
		System.out.println(msg);
	}

	/** Main method
	  */
	public static void main(String[] args)
	{
		if(args.length < 1)
		{
			System.out.println("USAGE : java DSTC zipfile");
			System.out.println("");
			System.out.println("DSTC (Death Star Trash Compactor) created by Howard A Kistler & protected by the GPL");
		}
		else
		{
			DSTC dstc = new DSTC(args[0]);
		}
	}

}